==> cmss-react-app-1.0.0.tgz: package/app/webpack.config.js <==
const path = require('path');
const webpack = require('webpack');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');

const HappyPack = require('happypack');
const os = require('os');
const happyThreadPool = HappyPack.ThreadPool({ size: os.cpus().length });

const host = process.env.HOST || 'localhost';
const sourcePath = path.join(__dirname, './src');
const staticPath = path.join(__dirname, '../static');

const nodeEnv = process.env.NODE_ENV || 'development';

const isProd = nodeEnv.indexOf('production') !== -1;

const isMock = nodeEnv.indexOf('mock') !== -1;
const port = isMock? 3005 : 3006;

const isProdDebugAllowed = false; // 生产模式亦可调试代码

const plugins = [
  new MiniCssExtractPlugin({
    filename: 'stylesheets/[name].css',
  }),
  new webpack.DefinePlugin({
    'process.env': {
      NODE_ENV: JSON.stringify(nodeEnv.replace(/(^\s*)|(\s*$)/g, '')),
    },
  }),
  new webpack.ProvidePlugin({
    $: 'jquery',
    jQuery: 'jquery',
    'window.jQuery': 'jquery',
  }),
  new HappyPack({
    id: 'happy-babel',
    loaders: ['babel-loader?cacheDirectory=true'],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new HappyPack({
    id: 'happy-eslint',
    loaders: ['eslint-loader'],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new HappyPack({
    id: 'happy-css',
    loaders: ['style-loader', 'css-loader'],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new HappyPack({
    id: 'happy-less',
    loaders: [
      'style-loader',
      'css-loader',
      {
        loader: 'less-loader',
        options: {
          javascriptEnabled: true,
        },
      },
    ],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new webpack.LoaderOptionsPlugin({
    options: {}
  }),
];

if (isProd) {
  plugins.unshift(new CleanWebpackPlugin([staticPath], {
    allowExternal: true,
    verbose: true,
    dry: false,
  }));
} else {
  plugins.push(
    new webpack.HotModuleReplacementPlugin()
  );
}

const autoprefixerLoader = {
  loader: 'postcss-loader',
  options: {
    ident: 'postcss',
    plugins: [
      require('autoprefixer')({broswer: 'last 5 versions'}),
    ]
  }
};
const loaders = [
  {
    test: /\.html$/,
    use: [
      {
        loader: 'file-loader?name=../[name].[ext]',
        query: {
          name: '[name].[ext]?[hash]',
        },
      },
    ],
  },
  {
    test: /\.(js|jsx)$/,
    exclude: /node_modules/,
    use: isProd ? ['happypack/loader?id=happy-babel']
      : ['happypack/loader?id=happy-babel', 'happypack/loader?id=happy-eslint'],
  },
  {
    test: /\.(jpg|png|gif?)$/,
    loader: isProd ? 'url-loader?limit=8192&name=stylesheets/images/[name].[ext]?[hash]' : 'url-loader?limit=8192',
  },
  {
    test: /\.(ttf|eot|svg|woff|otf?)$/,
    loader: 'file-loader?&name=stylesheets/font/[name].[ext]?[hash]',
  },
  {
    test: /\.ico$/,
    loader: 'file-loader?name=[name].[ext]?[hash]',
  },
  {
    test: /\.css?$/,
    use: isProd
      ? [MiniCssExtractPlugin.loader, 'css-loader', autoprefixerLoader]
      : ['happypack/loader?id=happy-css'],
  },
  {
    test: /\.less$/,
    use: isProd
      ? [
        MiniCssExtractPlugin.loader,
        'css-loader',
        autoprefixerLoader,
        {
          loader: 'less-loader',
          options: {
            javascriptEnabled: true,
          },
        }
      ]
      : ['happypack/loader?id=happy-less'],
  },
];

const config = {
  cache: true,
  mode: isProd ? 'production' : 'development' ,
  devtool: isProd ? (isProdDebugAllowed ? 'source-map' : '') : 'eval',
  entry: {
    bundle: ['@babel/polyfill', './src/index'],
    vendors: ['react', 'react-dom', 'react-router', 'react-redux', 'redux', 'redux-thunk', 'jquery'],
  },
  output: {
    path: staticPath,
    filename: 'javascripts/[name].js',
    chunkFilename: 'javascripts/[name].chunk.js',
    publicPath: '/static/',
  },
  optimization: {
    splitChunks: {
      chunks: "initial",
      minSize: 30000,
      minChunks: 2,
      maxAsyncRequests: 1,
      maxInitialRequests: 1,
      name: false,
      cacheGroups: {
        vendor: {
          priority: -10,
          chunks: "initial",
          test: /[\\/]node_modules[\\/]/,
          name: "vendor",
          minSize: 30000,
          minChunks: 2,
          enforce: true,
          maxAsyncRequests: 1,
          maxInitialRequests: 1,
          reuseExistingChunk: true
        },
      }
    },
    minimize: isProd,
    minimizer: [
      new TerserPlugin({
        exclude: /\.min\.js$/,
        cache: true,
        parallel: true,
        sourceMap: isProdDebugAllowed,
        terserOptions: {
          ecma: undefined,
          warnings: false,
          compress: {
            drop_console: true,
            drop_debugger: true,
            keep_infinity: true,
          },
          mangle: false,
          output: {
            comments: false,
          },
          ie8: false,
          comments: false,
        },
      }),
    ],
  },
  plugins,
  node: {
    fs: 'empty',
  },
  module: {
    rules: loaders,
  },
  resolve: {
    extensions: ['.js', '.jsx'],
    modules: [sourcePath, 'node_modules'],
  },
  externals: {
    BMap: 'BMap',
    BMapLib: 'BMapLib',
  },
  devServer: {
    open: true,
    inline: !isProd,
    hot: !isProd,
    contentBase: './src',
    historyApiFallback: true,
    watchContentBase: true,
    host,
    port,
    compress: isProd,
    stats: {
      assets: true,
      children: false,
      chunks: false,
      hash: false,
      modules: false,
      publicPath: false,
      timings: true,
      version: false,
      warnings: true,
      colors: {
        green: '\u001b[32m',
      },
    },
  },
};

module.exports = config;

==> cmss-react-app-2.0.1.tgz: package/app/webpack.config.js <==
const path = require('path');
const webpack = require('webpack');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');

const HappyPack = require('happypack');
const os = require('os');
const happyThreadPool = HappyPack.ThreadPool({ size: os.cpus().length });

const host = process.env.HOST || 'localhost';
const sourcePath = path.join(__dirname, './src');
const staticPath = path.join(__dirname, '../static');

const nodeEnv = process.env.NODE_ENV || 'development';

const isProd = nodeEnv.indexOf('production') !== -1;
const isMock = nodeEnv.indexOf('mock') !== -1;

const port = isMock? 3005 : 3006;

const publicPath = '/static/';

const isProdDebugAllowed = false; // 生产模式是否可调试代码

const plugins = [
  new MiniCssExtractPlugin({
    filename: 'stylesheets/[name].css',
  }),
  new webpack.DefinePlugin({
    'process.env': {
      NODE_ENV: JSON.stringify(nodeEnv.replace(/(^\s*)|(\s*$)/g, '')),
    },
  }),
  new webpack.ProvidePlugin({
    $: 'jquery',
    jQuery: 'jquery',
    'window.jQuery': 'jquery',
  }),
  new HappyPack({
    id: 'happy-babel',
    loaders: ['babel-loader?cacheDirectory=true'],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new HappyPack({
    id: 'happy-eslint',
    loaders: ['eslint-loader'],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new HappyPack({
    id: 'happy-css',
    loaders: ['style-loader', 'css-loader'],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new HappyPack({
    id: 'happy-less',
    loaders: [
      'style-loader',
      'css-loader',
      {
        loader: 'less-loader',
        options: {
          javascriptEnabled: true,
        },
      },
    ],
    threadPool: happyThreadPool,
    verbose: false,
  }),
  new webpack.LoaderOptionsPlugin({
    options: {}
  }),
];

if (isProd) {
  plugins.unshift(new CleanWebpackPlugin({
    cleanOnceBeforeBuildPatterns: [staticPath],
    dangerouslyAllowCleanPatternsOutsideProject: true,
    verbose: true,
    dry: false,
  }));
} else {
  plugins.push(
    new webpack.HotModuleReplacementPlugin()
  );
}

const autoprefixerLoader = {
  loader: 'postcss-loader',
  options: {
    ident: 'postcss',
    plugins: [
      require('autoprefixer')({broswer: 'last 5 versions'}),
    ]
  }
};
const loaders = [
  {
    test: /\.html$/,
    use: [
      {
        loader: 'file-loader?name=../[name].[ext]',
        query: {
          name: '[name].[ext]?[hash]',
        },
      },
    ],
  },
  {
    test: /\.(js|jsx)$/,
    exclude: /node_modules/,
    use: isProd ? ['happypack/loader?id=happy-babel']
      : ['happypack/loader?id=happy-babel', 'happypack/loader?id=happy-eslint'],
  },
  {
    test: /\.(jpg|png|gif?)$/,
    loader: isProd ? 'url-loader?limit=8192&name=stylesheets/images/[name].[ext]?[hash]' : 'url-loader?limit=8192',
  },
  {
    test: /\.(ttf|eot|svg|woff|otf?)$/,
    loader: 'file-loader?&name=stylesheets/font/[name].[ext]?[hash]',
  },
  {
    test: /\.ico$/,
    loader: 'file-loader?name=[name].[ext]?[hash]',
  },
  {
    test: /\.css?$/,
    use: isProd
      ? [MiniCssExtractPlugin.loader, 'css-loader', autoprefixerLoader]
      : ['happypack/loader?id=happy-css'],
  },
  {
    test: /\.less$/,
    use: isProd
      ? [
        MiniCssExtractPlugin.loader,
        'css-loader',
        autoprefixerLoader,
        {
          loader: 'less-loader',
          options: {
            javascriptEnabled: true,
          },
        }
      ]
      : ['happypack/loader?id=happy-less'],
  },
];

const config = {
  cache: true,
  mode: isProd ? 'production' : 'development' ,
  devtool: isProd ? (isProdDebugAllowed ? 'source-map' : '') : 'eval',
  entry: {
    bundle: ['@babel/polyfill', './src/index'],
    vendors: ['react', 'react-dom', 'react-router-dom', 'react-redux', 'redux', 'redux-thunk', 'jquery'],
  },
  output: {
    path: staticPath,
    filename: 'javascripts/[name].js',
    chunkFilename: 'javascripts/[name].chunk.js',
    publicPath,
  },
  optimization: {
    splitChunks: {
      chunks: "initial",
      minSize: 30000,
      minChunks: 2,
      maxAsyncRequests: 1,
      maxInitialRequests: 1,
      name: false,
      cacheGroups: {
        vendor: {
          priority: -10,
          chunks: "initial",
          test: /[\\/]node_modules[\\/]/,
          name: "vendor",
          minSize: 30000,
          minChunks: 2,
          enforce: true,
          maxAsyncRequests: 1,
          maxInitialRequests: 1,
          reuseExistingChunk: true
        },
      }
    },
    minimize: isProd,
    minimizer: [
      new TerserPlugin({
        exclude: /\.min\.js$/,
        cache: true,
        parallel: true,
        sourceMap: isProdDebugAllowed,
        terserOptions: {
          ecma: undefined,
          warnings: false,
          compress: {
            drop_console: true,
            drop_debugger: true,
            keep_infinity: true,
          },
          mangle: false,
          output: {
            comments: false,
          },
          ie8: false,
          comments: false,
        },
      }),
    ],
  },
  plugins,
  node: {
    fs: 'empty',
  },
  module: {
    rules: loaders,
  },
  resolve: {
    extensions: ['.js', '.jsx'],
    modules: [sourcePath, 'node_modules'],
  },
  devServer: {
    open: true,
    inline: !isProd,
    hot: !isProd,
    contentBase: './src',
    historyApiFallback: true,
    watchContentBase: true,
    host,
    port,
    compress: isProd,
    stats: {
      assets: true,
      children: false,
      chunks: false,
      hash: false,
      modules: false,
      publicPath: false,
      timings: true,
      version: false,
      warnings: true,
      colors: {
        green: '\u001b[32m',
      },
    },
  },
};

module.exports = config;
